#include "stdafx.h"
#include "clf_resources.h"
#include "clf_skybox.h"
#include "clf_util.h"

clf_skybox::clf_skybox(float b_size/*=2.f*/)
{
	for (int idx = 0; idx < 6; idx++)
		m_textures[idx] = 0;
	float boxsize;
	boxsize = b_size;
	m_size = clf_vector3D (boxsize, boxsize, boxsize);
}

clf_skybox::~clf_skybox()
{
	release();
}

bool clf_skybox::load_textures(const char* top, const char* bottom, const char* front, const char* back, const char* left, const char* right)
{
	m_textureNames[SKY_TOP] = std::string(top);
	m_textureNames[SKY_BOTTOM] = std::string(bottom);
	m_textureNames[SKY_EAST] = std::string(front);
	m_textureNames[SKY_WEST] = std::string(back);
	m_textureNames[SKY_NORTH] = std::string(left);
	m_textureNames[SKY_SOUTH] = std::string(right);

	clf_reset();

	return true;
}

void clf_skybox::clf_reset()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);

	m_textures[SKY_TOP] = clf_loadTexture(m_textureNames[SKY_TOP].c_str());
	m_textures[SKY_BOTTOM] = clf_loadTexture(m_textureNames[SKY_BOTTOM].c_str());
	m_textures[SKY_EAST] = clf_loadTexture(m_textureNames[SKY_EAST].c_str());
	m_textures[SKY_WEST] = clf_loadTexture(m_textureNames[SKY_WEST].c_str());
	m_textures[SKY_NORTH] = clf_loadTexture(m_textureNames[SKY_NORTH].c_str());
	m_textures[SKY_SOUTH] = clf_loadTexture(m_textureNames[SKY_SOUTH].c_str());

	glPopAttrib();

	draw_list = glGenLists(1);
	glNewList(draw_list, GL_COMPILE);
	build_draw_list();
	glEndList();
}

void clf_skybox::build_draw_list()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_DEPTH_TEST);
	glDisable (GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_TEXTURE_CUBE_MAP_SEAMLESS);

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	float padd;
	padd = 2.0e-4f;

	//	uncommenting this will make a perfect sky box texture stitch, unfortunately
	//  this option ios not supported by all version of opengl	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);


	// the cube will just be drawn as six quads for the sake of simplicity
	// for each face, we specify the quad's normal (for lighting), then
	// specify the quad's 4 vertices's and associated texture coordinates

	glBindTexture(GL_TEXTURE_2D, m_textures[SKY_EAST]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f + padd, 1.0f - padd); glVertex3f(-m_size.x,  m_size.y, m_size.z);
	glTexCoord2f(1.0f - padd, 1.0f - padd); glVertex3f( m_size.x,  m_size.y, m_size.z);
	glTexCoord2f(1.0f - padd, 0.0f + padd); glVertex3f( m_size.x, -m_size.y, m_size.z);
	glTexCoord2f(0.0f + padd, 0.0f + padd); glVertex3f(-m_size.x, -m_size.y, m_size.z);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, m_textures[SKY_NORTH]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f + padd, 1.0f - padd); glVertex3f(-m_size.x,  m_size.y, -m_size.z);
	glTexCoord2f(1.0f - padd, 1.0f - padd); glVertex3f(-m_size.x,  m_size.y,  m_size.z);
	glTexCoord2f(1.0f - padd, 0.0f + padd); glVertex3f(-m_size.x, -m_size.y,  m_size.z);
	glTexCoord2f(0.0f + padd, 0.0f + padd); glVertex3f(-m_size.x, -m_size.y, -m_size.z);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, m_textures[SKY_WEST]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f + padd, 1.0f - padd); glVertex3f( m_size.x,  m_size.y, -m_size.z);
	glTexCoord2f(1.0f - padd, 1.0f - padd); glVertex3f(-m_size.x,  m_size.y, -m_size.z);
	glTexCoord2f(1.0f - padd, 0.0f + padd); glVertex3f(-m_size.x, -m_size.y, -m_size.z);
	glTexCoord2f(0.0f + padd, 0.0f + padd); glVertex3f( m_size.x, -m_size.y, -m_size.z);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, m_textures[SKY_SOUTH]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f + padd, 1.0f - padd); glVertex3f(m_size.x,   m_size.y,  m_size.z);
	glTexCoord2f(1.0f - padd, 1.0f - padd); glVertex3f(m_size.x,   m_size.y, -m_size.z);
	glTexCoord2f(1.0f - padd, 0.0f + padd); glVertex3f(m_size.x,  -m_size.y, -m_size.z);
	glTexCoord2f(0.0f + padd, 0.0f + padd); glVertex3f(m_size.x,  -m_size.y,  m_size.z);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, m_textures[SKY_TOP]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f + padd, 1.0f - padd); glVertex3f(-m_size.x,  m_size.y, -m_size.z);
	glTexCoord2f(1.0f + padd, 1.0f + padd); glVertex3f( m_size.x,  m_size.y, -m_size.z);
	glTexCoord2f(1.0f - padd, 0.0f + padd); glVertex3f( m_size.x,  m_size.y,  m_size.z);
	glTexCoord2f(0.0f - padd, 0.0f - padd); glVertex3f(-m_size.x,  m_size.y,  m_size.z);
	glEnd();	 

	glBindTexture(GL_TEXTURE_2D, m_textures[SKY_BOTTOM]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f + padd, 1.0f + padd); glVertex3f(-m_size.x, -m_size.y,  m_size.z);
	glTexCoord2f(1.0f + padd, 1.0f - padd); glVertex3f( m_size.x, -m_size.y,  m_size.z);
	glTexCoord2f(1.0f - padd, 0.0f - padd); glVertex3f( m_size.x, -m_size.y, -m_size.z);
	glTexCoord2f(0.0f - padd, 0.0f + padd); glVertex3f(-m_size.x, -m_size.y, -m_size.z);
	glEnd();	 	

	glPopAttrib();
}

void clf_skybox::render(float cameraX, float cameraY, float cameraZ)
{
	glPushMatrix();

	// Move the skybox so that it's centered on the camera.
	glTranslatef(cameraX, cameraY, cameraZ);

	glCallList(draw_list);

	glPopMatrix();
}

void clf_skybox::release()
{
}
